MODULE Subface

CONTAINS

  SUBROUTINE Initialize_Subface_Module()
    IMPLICIT NONE
  END SUBROUTINE

!
! Incluye todas las subrutinas que utilizan las programas de dinamica de
! superficies
!
!
!**************************** D I F S Y *******************************
!                                                             
!       Traduction du programme ALGOL de Bulirsch and Stoer, Numerische
!     Mathematik 8(1966)1.
!       DIFSY calcule un pas d'integration H inferieur ou egal a H0 pour
!     une equation differentielle ordinaire du premier ordre de forme:
!     dZ/dX=F(X,Z)
!     dont le membre de droite doit etre donne par un sous-programme
!     commencant par:
!     SUBROUTINE F(X,Z,DZ)
!       Le programme prend pour pas H le premier des nombres H0, H0/2,
!     H0/4 pour lequel 9 etapes d'extrapolation suffisent a obtenir le
!     resultat avec la precision requise. Si H est different de H0, le
!     parametre FIN prend la valeur .TRUE.
!       X et Y sont les valeurs initiales. Apres execution de DIFSY, les
!     valeurs initiales de X et Y sont remplacees par X+H et Y(X+H). H0
!     peut egalement etre modifie (changement de pas automatique). La
!     nouvelle valeur de H0 est la meilleure valeur estimee pour le pas
!     suivant.
!       S et la constante EPS commandent la precision des valeurs
!     calculees. Le calcul est arrete lorsque deux valeurs successives
!     de Y different au plus de EPS*S. EPS ne doit pas etre inferieur a
!     10**(-D+3) ou D est le nombre de chiffres decimaux dans la machine
!     utilisee.
!         Il est conseille de faire S=0. pour le premier pas
!     d'integration. A la sortie de DIFSY, S est devenu S=MAX(S,Y(XSI))
!     pour XSI compris dans l'intervalle (X,X+H).
!     *
  SUBROUTINE DIFSY(x,y,h0,eps,s,f,fin)
    IMPLICIT NONE

    INTEGER          :: r, sr
    DOUBLE PRECISION :: dt(7), d(7), yg(8), yh(8)
    DOUBLE PRECISION :: a, fc, g, h0, s, x, eps
    DOUBLE PRECISION :: y, dz, ya, yl, ym, u, v, ta, c, b1, b, dy
    LOGICAL          :: konv, bo, bh, fin
    INTEGER          :: m, jjp1, jp1, lp1, kk, k, kp1

    CALL F(x,y,dz)
    bh = .FALSE.
    fin = .FALSE.
    ya = y
 20 CONTINUE
    a = h0 + x
    fc = 1.5D0
    bo = .FALSE.
    m = 1
    r = 2
    sr = 3
    jjp1 = 0

!   Extrapolations successives.
    DO 170 jp1 = 1, 10
      IF (bo) THEN
        d(2) = 16.D0 / 9.D0
        d(4) = 64.D0 / 9.D0
        d(6) = 256.D0 / 9.D0
      ELSE
        d(2) = 9.D0 / 4.D0
        d(4) = 9.D0
        d(6) = 36.D0
      END IF
      konv = .FALSE.
      IF (jp1 .GT. 3) konv = .TRUE.
      IF (jp1 .GT. 7) THEN
        lp1 = 7
        fc = 0.6D0 * fc
        d(7) = 64.D0
      ELSE
        lp1 = jp1
        d(LP1) = DFLOAT(m * m)
      END IF
      m = 2 * m
      g = h0 / DFLOAT(m)
      b = 2.D0 * g
      IF (bh .AND. jp1 .LT. 9) THEN
        ym = yh(jp1)
        yl = yg(jp1)
      ELSE
        kk = (m - 2) / 2

!       Calcul de T(H)
        m = m - 1
        yl = ya
        ym = ya + g * dz
        DO 100 k = 1, m
          CALL F(x + DFLOAT(k) * g, ym, dy)
          u = yl + b * dy
          yl = ym
          ym = u
          u = DABS(u)
          IF (u .GT. s) s=u
          IF (k .NE. kk .OR. k .EQ. 2) GO TO 100
          jjp1 = 1 + jjp1
          yh(jjp1) = ym
          yg(jjp1) = yl
100     CONTINUE
      END IF
      CALL F(a, ym, dy)
      v = dt(1)
      ta = (ym + yl + g * dy) / 2.D0

!     Calcul de la valeur extrapolee T(0).
      c = ta
      dt(1) = ta
      IF (lp1 .GE. 2) THEN
        DO 150 kp1 = 2, lp1
          b1 = d(kp1) * v
          b = b1 - c
          u = v
          IF (b .NE. 0.D0) THEN
            b = (c - v) / b
            u = c * b
            c = b1 * b
          END IF
          v = dt(kp1)
          dt(kp1) = u
          ta = u + ta
  150   CONTINUE
      END IF
      IF (DABS(y - ta) .GT. eps * s) konv = .FALSE.
      y = ta
      IF (konv) GO TO 180
      d(3) = 4.D0
      d(5) = 16.D0
      bo = .NOT. bo
      m = r
      r = sr
      sr = m * 2
170 CONTINUE

!   Plus de 9 iterations etaient necessaires. On recommence en
!   divisant le pas par 2.
    bh = .NOT. bh
    fin = .TRUE.
    h0 = h0 / 2.D0
    GO TO 20
180 h0 = fc * h0
    x = a
  END SUBROUTINE
!
!**************************** D I F S Y 2 ***************************
      SUBROUTINE DIFSY2(n,x,y,h0,eps,s,f,fin)
!     IMPLICIT NONE
! 
!       Version 0.1     29/1/97
!
!       Traduction du programme ALGOL de Bulirsch and Stoer, 
!     Numerische Mathematik 8(1966)1. Double precision.
!       DIFSY2 calcule un pas d'integration H inferieur ou egal a H0 
!     pour un systeme de N equations differentielles ordinaires du 
!     premier ordre de forme:
!     dY/dX=F(X,Y)
!     dont le membre de droite doit etre donne par un sous-programme
!     commencant par:
!     SUBROUTINE F(x,y,dy)
!     REAL*8 y(n),dy(n)
!     et dont le nom est passe comme argument (f) dans la sequence
!     d'appel de DIFSY2.
!       Le programme prend pour pas H le premier des nombres H0, H0/2,
!     H0/4 pour lequel 9 etapes d'extrapolation suffisent a obtenir le
!     resultat avec la precision requise. Si H est different de H0, le
!     parametre FIN prend la valeur .TRUE.
!       X et Y sont les valeurs initiales. Apres execution de DIFSY2, 
!     les valeurs initiales de X et Y sont remplacees par X+H et 
!     Y(X+H). H0 peut egalement etre modifie (changement de pas 
!     automatique). La nouvelle valeur de H0 est la meilleure valeur 
!     estimee pour le pas suivant.
!       Le tableau S et la constante EPS commandent la precision des 
!     valeurs calculees. Le calcul est arrete lorsque deux valeurs 
!     successives de Y(i) different au plus de EPS*S(i). EPS ne doit 
!     pas etre inferieur a 10**(-D+3) ou D est le nombre de chiffres 
!     decimaux dans la machine utilisee.
!       Il est conseille de faire S(i)=0. pour le premier pas
!     d'integration. A la sortie de DIFSY2, S(i) est devenu 
!     S(i)=MAX[S(i),Y(i,xsi)] 
!     pour xsi compris dans l'intervalle [X,X+H].
!     *
      INTEGER id,i,m,n
      PARAMETER (id=99)
      DOUBLE PRECISION d2,d4,d6,dp2
      PARAMETER(d2=16.D0/9.D0,d4=64.D0/9.D0,d6=256.D0/9.D0,dp2=9.D0/4.D0)

      INTEGER r,sr
      LOGICAL bo,bh,fin,konv
      DOUBLE PRECISION d(7),dt(id,7),dy(id),dz(id),s(:),y(:),ya(id),yg(8,id),yh(8,id),yl(id),ym(id)
      DOUBLE PRECISION a,b1,b,c,eps,fc,g,h0,ta,u,v,x
      INTEGER jjp1,jp1,lp1,kk,k,kp1

      IF(n.GT.id) THEN
        WRITE(*,*) 'ID too small in DIFSY2'
        STOP
      END IF

      CALL F(x,y,dz)
      bh=.FALSE.
      fin=.FALSE.
      DO i=1,n
        ya(i)=y(i)
      END DO
!                                                          Iterations
   20 CONTINUE
      a=h0+x
      fc=1.5D0
      bo=.FALSE.
      m=1
      r=2
      sr=3
      jjp1=0
!                                          Extrapolations successives
      DO 170 jp1=1,10
        IF(bo) THEN
          d(2)=d2
          d(4)=d4
          d(6)=d6
        ELSE
          d(2)=dp2
          d(4)=9.D0
          d(6)=36.D0
        END IF
        konv=.FALSE.
        IF(jp1.GT.3) konv=.TRUE.
        IF(jp1.GT.7) THEN
          lp1=7
          fc=0.6D0*fc
          d(7)=64.D0
        ELSE
          lp1=jp1
          d(lp1)=DFLOAT(m**2)
        END IF
        m=2*m
        g=h0/DFLOAT(m)
        b=2.D0*g
        IF(bh.AND.jp1.LT.9) THEN
          DO i=1,n
            ym(i)=yh(jp1,i)
            yl(i)=yg(jp1,i)
          END DO
        ELSE
!                                                      Calcul de T(H)
          kk=(m-2)/2
          m=m-1
          DO i=1,n
            yl(i)=ya(i)
            ym(i)=ya(i)+g*dz(i)
          END DO
          DO k=1,m
            CALL F(x+DFLOAT(k)*g,ym,dy)
            DO i=1,N
              u=yl(i)+b*dy(i)
              yl(i)=ym(i)
              ym(i)=u
              u=DABS(u)
              IF(u.GT.s(i))  s(i)=u
            END DO
            IF(k.NE.kk.OR.k.EQ.2) GO TO 100
            jjp1=1+jjp1
            DO i=1,n
              yh(jjp1,i)=ym(i)
              yg(jjp1,i)=yl(i)
            END DO
 100        CONTINUE
          END DO
        END IF
        CALL F(a,ym,dy)
        DO 160 i=1,n
          v=dt(i,1)
          ta=(ym(i)+yl(i)+g*dy(i))/2.D0
!                                 Calcul de la valeur extrapolee T(0)
          c=ta
          dt(i,1)=ta
          IF(lp1.GE.2) THEN     
            DO kp1=2,lp1
              b1=d(kp1)*v
              b=b1-c
              u=v
              IF(b.NE.0.D0) THEN     
                b=(c-v)/b
                u=c*b
                c=b1*b
              END IF
              v=dt(i,kp1)
              dt(i,kp1)=u
              ta=u+ta
            END DO  
          END IF
          IF(DABS(y(i)-ta).GT.eps*s(i)) konv=.FALSE.
          y(i)=ta
 160    CONTINUE          
        IF(konv) GO TO 180
        d(3)=4.D0
        d(5)=16.D0
        bo=.NOT.bo
        m=r
        r=sr
        sr=m*2
 170  CONTINUE
!
!     Plus de 9 iterations etaient necessaires. On recommence en
!     divisant le pas par 2.
!     
      bh=.NOT.bh
      fin=.TRUE.
      h0=h0/2.D0
      GO TO 20
! 
 180  h0=fc*h0
      x=a
      RETURN
      END SUBROUTINE
!
!**************************** D I F S Y 2 S *************************
      SUBROUTINE DIFSY2S(n,x,y,h0,eps,s,f,switch) !,ideal,distorted,surfvecnum)
! 
!       Version 0.1     29/1/97
!
!       Traduction du programme ALGOL de Bulirsch and Stoer, 
!     Numerische Mathematik 8(1966)1. Double precision.
!       Cette version inclue un switch logique (SWITCH) determin par 
!     la routine F qui arrete l'integration quand SWITCH prend la 
!     valeur .TRUE. 
!       DIFSY2S calcule un pas d'integration H inferieur ou egal a H0 
!     pour un systeme de N equations differentielles ordinaires du 
!     premier ordre de forme:
!     dY/dX=F(X,Y)
!     dont le membre de droite doit etre donne par un sous-programme
!     commencant par:
!     SUBROUTINE F(x,y,dy,switch)
!     REAL*8 y(n),dy(n)
!     et dont le nom est passe comme argument (f) dans la sequence
!     d'appel de DIFSY2.
!       Le programme prend pour pas H le premier des nombres H0, H0/2,
!     H0/4 pour lequel 9 etapes d'extrapolation suffisent a obtenir le
!     resultat avec la precision requise.
!       X et Y sont les valeurs initiales. Apres execution de DIFSY2, 
!     les valeurs initiales de X et Y sont remplacees par X+H et 
!     Y(X+H). H0 peut egalement etre modifie (changement de pas 
!     automatique). La nouvelle valeur de H0 est la meilleure valeur 
!     estimee pour le pas suivant.
!       Le tableau S et la constante EPS commandent la precision des 
!     valeurs calculees. Le calcul est arrete lorsque deux valeurs 
!     successives de Y(i) different au plus de EPS*S(i). EPS ne doit 
!     pas etre inferieur a 10**(-D+3) ou D est le nombre de chiffres 
!     decimaux dans la machine utilisee.
!       Il est conseille de faire S(i)=0. pour le premier pas
!     d'integration. A la sortie de DIFSY2, S(i) est devenu 
!     S(i)=MAX[S(i),Y(i,xsi)] 
!     pour xsi compris dans l'intervalle [X,X+H].
!     *
!     IMPLICIT NONE

      INTEGER id
      PARAMETER (id=99)
      DOUBLE PRECISION d2,d4,d6,dp2
      PARAMETER(d2=16.D0/9.D0,d4=64.D0/9.D0,d6=256.D0/9.D0,dp2=9.D0/4.D0)
!
      INTEGER r,sr
      LOGICAL bo,bh,konv,switch
      DOUBLE PRECISION d(7),dt(id,7),dy(id),dz(id),s(:),y(:),ya(id), &
                       yg(8,id),yh(8,id),yl(id),ym(id),a,b1,b,c,eps,fc, &
                       g,h0,ta,u,v,x

!     BEGIN PHONON VARIABLES/FUNCTIONS
!      DOUBLE PRECISION ideal(1:10000,1:3)
!      DOUBLE PRECISION distorted(1:10000,1:3)
!      INTEGER surfvecnum
!     END PHONON

      INTEGER n,i,m,jjp1,jp1,lp1,kk,k,kp1
!
      IF(n.GT.id) THEN
        WRITE(*,*) 'ID too small in DIFSYS'
        STOP
      END IF

      switch=.FALSE.
      CALL F(x,y,dz,switch,ideal,distorted,surfvecnum)
      IF(switch) RETURN
      bh=.FALSE.
      DO i=1,n
        ya(i)=y(i)
      END DO
!                                                          Iterations
   20 CONTINUE
      a=h0+x
      fc=1.5D0
      bo=.FALSE.
      m=1
      r=2
      sr=3
      jjp1=0
!                                          Extrapolations successives
      DO 170 jp1=1,10
        IF(bo) THEN    
          d(2)=d2
          d(4)=d4
          d(6)=d6
        ELSE
          d(2)=dp2
          d(4)=9.D0
          d(6)=36.D0
        END IF
        konv=.FALSE.
        IF(jp1.GT.3) konv=.TRUE.
        IF(jp1.GT.7) THEN    
          lp1=7
          fc=0.6D0*fc
          d(7)=64.D0
        ELSE
          lp1=jp1
          d(lp1)=DFLOAT(m**2)
        END IF
        m=2*m
        g=h0/DFLOAT(m)
        b=2.D0*g
        IF(bh.AND.jp1.LT.9) THEN
          DO i=1,n
            ym(i)=yh(jp1,i)
            yl(i)=yg(jp1,i)
          END DO
        ELSE
!                                                      Calcul de T(H)
          kk=(m-2)/2
          m=m-1
          DO  i=1,n
            yl(i)=ya(i)
            ym(i)=ya(i)+g*dz(i)
          END DO
          DO k=1,m
            CALL F(x+DFLOAT(k)*g,ym,dy,switch,ideal,distorted,surfvecnum)
            IF(switch) RETURN
            DO i=1,N
              u=yl(i)+b*dy(i)
              yl(i)=ym(i)
              ym(i)=u
              u=DABS(u)
              IF(u.GT.s(i))  s(i)=u
            END DO  
            IF(k.NE.kk.OR.k.EQ.2) GO TO 100
            jjp1=1+jjp1
            DO i=1,n
              yh(jjp1,i)=ym(i)
              yg(jjp1,i)=yl(i)
            END DO  
 100        CONTINUE
          END DO  
        END IF   
        CALL F(a,ym,dy,switch,ideal,distorted,surfvecnum)
        IF(switch) RETURN
        DO 160 i=1,n
          v=dt(i,1)
          ta=(ym(i)+yl(i)+g*dy(i))/2.D0
!                                 Calcul de la valeur extrapolee T(0)
          c=ta
          dt(i,1)=ta
          IF(lp1.GE.2) THEN     
            DO kp1=2,lp1
              b1=d(kp1)*v
              b=b1-c
              u=v
              IF(b.NE.0.D0) THEN     
                b=(c-v)/b
                u=c*b
                c=b1*b
              END IF
              v=dt(i,kp1)
              dt(i,kp1)=u
              ta=u+ta
            END DO  
          END IF
          IF(DABS(y(i)-ta).GT.eps*s(i)) konv=.FALSE.
          y(i)=ta
 160    CONTINUE          
        IF(konv) GO TO 180
        d(3)=4.D0
        d(5)=16.D0
        bo=.NOT.bo
        m=r
        r=sr
        sr=m*2
 170  CONTINUE
!
!     Plus de 9 iterations etaient necessaires. On recommence en
!     divisant le pas par 2.
!     
      bh=.NOT.bh
      h0=h0/2.D0
      GO TO 20
! 
 180  h0=fc*h0
      x=a
      RETURN
      END SUBROUTINE
!
!
!**************************** S A K E N *****************************
!        SAKEN   VERSION 6.2   31/10/75 - 5/1/00      A. SALIN
!
!        Zero of a function
!
!        Double precision
!
!        Start with an upper (RSUP) and lower (RINF) bound of the
!        root.
!        Firstly, the interval [RINF,RSUP] is decreased such that
!        |RSUP-RINF| < DR. Then the root is found with precision EPS
!        by successive interpolations using Aitken scheme.
!
!        Constraints:
!             The first part assumes that the root is not located at
!             an extremum of the function.
!        The function is given in a FUNCTION.
!
!        RINF: lower bound on R.
!        RSUP: upper bound on R.
!        DR: interval between RINF and RSUP before starting
!        interpolation.
!        EPS: required precision on R.
!        F: name of FUNCTION procedure defining the function.
!        R: set to the root position.
!        KOD:
!                  On input:
!             KOD=0 : do interpolation even if the root is not
!        between RINF and RSUP after decreasing the interval to DR.
!             KOD=-1 : message written if the root is not in the
!        defined interval.
!                  On output:
!             KOD=0 : convergence obtained.
!             KOD=1 : no single root between RINF and RSUP
!             KOD=2 : interpolation has not converged
!        ICONS: unit for error messages
!
      SUBROUTINE SAKEN(rinf,rsup,dr,eps,F,r,kod,icons)
      IMPLICIT NONE
      INTEGER nmax
      PARAMETER (nmax=20)
      DOUBLE PRECISION rsup,rinf,dr,eps,r1,r2,r3,y1,y2,y3,ext,r,xx,F,prod,a(nmax),x(nmax)
      INTEGER kod,icons,n,nm,i
!
      r1=rinf
      r2=rsup
      y1=F(r1)
      IF(y1.EQ.0.d0) THEN
        r=r1
        kod=0
        RETURN
      END IF
      y2=F(r2)
      IF(y2.EQ.0.d0) THEN
        r=r2
        kod=0
        RETURN
      END IF
!                                                     Reduce interval
   11 CONTINUE
      IF(DABS(r2-r1).GT.DABS(dr))  THEN
        r3=(r1+r2)/2.D0
        y3=F(r3)
        IF(y3.EQ.0.d0) THEN
          r=r3
          kod=0
          RETURN
        END IF
        IF((y3*y1).LE.0.D0) THEN
          y2=y3
          r2=r3
        ELSE
          y1=y3
          r1=r3
        END IF
      GO TO 11
      END IF
!                                              Check location of root
      IF(y1*y2.GT.0.D0) THEN
        IF(kod.EQ.-1) THEN
          WRITE(icons,3) r1,y1,r2,y2
    3 FORMAT('SAKEN: Root not between RINF and RSUP',/,' R1=',1PD12.6,3X,'Y1=',D12.6,3X,'R2=',D12.6,3X,'Y2=',D12.6)
          kod=1
          RETURN
        ELSE IF(kod.NE.0) THEN
          kod=1
          RETURN
        ELSE
          kod=1
        END IF
      END IF
!                                                       Interpolation
      a(1)=r1
      x(1)=y1
      ext=r1
      x(2)=y2
      r=(r2-r1)/(y2-y1)
      a(2)=r
      prod=-y1
      r=ext+prod*r
      IF(DABS(r-ext).LE.(DABS(r)*eps)) THEN
        IF(kod.NE.1) kod=0
        RETURN
      END IF
      ext=r
      DO n=3,nmax
        nm=n-1
        x(n)=F(r)
        xx=x(n)
        DO i=1,nm
          r=(r-a(i))/(xx-x(i))
        END DO
        a(n)=r
        prod=-prod*x(nm)
        r=ext+prod*r
      IF(DABS(r-ext).LE.(DABS(r)*eps)) THEN
        IF(kod.NE.1) kod=0
        RETURN
      END IF
        ext=r
      END DO
!                                            Convergence not achieved
      WRITE(icons,2) nmax
    2 FORMAT('SAKEN: more than ',i2,' points required')
      kod=2
      RETURN
      END SUBROUTINE
END MODULE

